package com.gic.haoban.manage.service.service.out.impl.hm;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.BasePageInfo;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.manage.api.dto.hm.HmLinkDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkStoreDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkStoreSettingDTO;
import com.gic.haoban.manage.api.dto.hm.HmLinkWxaDTO;
import com.gic.haoban.manage.api.dto.qdto.hm.HmLinkSearchQDTO;
import com.gic.haoban.manage.api.service.hm.HmLinkApiService;
import com.gic.haoban.manage.service.entity.hm.TabHmLink;
import com.gic.haoban.manage.service.errorcode.HaoBanErrCode;
import com.gic.haoban.manage.service.pojo.bo.hm.HmQrcodeBO;
import com.gic.haoban.manage.service.service.KeyDataService;
import com.gic.haoban.manage.service.service.hm.HmLinkService;
import com.gic.haoban.manage.service.service.hm.HmLinkStoreService;
import com.gic.haoban.manage.service.service.hm.HmQrcodeService;
import com.gic.member.api.dto.es.MemberDataDTO;
import com.gic.member.api.dto.es.MemberStoreClerkDataDTO;
import com.gic.member.api.service.MemberService;
import com.gic.search.engine.api.dto.ESResponseQuerySingle;
import com.gic.search.engine.api.dto.dynamic.DynamicSearchDTO;
import com.gic.search.engine.api.service.dynamic.ESDataDynamicOperationApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

@Service("hmLinkApiService")
public class HmLinkApiServiceImpl implements HmLinkApiService {

    private static final Logger log = LoggerFactory.getLogger(HmLinkApiService.class);

    @Autowired
    private HmLinkService hmLinkService;
    @Autowired
    private HmLinkStoreService hmLinkStoreService;
    @Autowired
    private KeyDataService keyDataService;
    @Autowired
    private MemberService memberService;
    @Autowired
    private ESDataDynamicOperationApiService esApiService;
    @Autowired
    private StoreService storeService;
    @Autowired
    private HmQrcodeService hmQrcodeService;
    @Autowired
    private ClerkService clerkService;

    @Override
    public ServiceResponse<HmLinkDTO> getByLinkId(String wxEnterpriseId, String enterpriseId, Long linkId) {
        TabHmLink link = this.hmLinkService.getByLinkId(wxEnterpriseId, enterpriseId, linkId);
        if (null != link) {
            HmLinkDTO dto = EntityUtil.changeEntityByJSON(HmLinkDTO.class, link);
            List<HmLinkStoreDTO> storeList = this.hmLinkStoreService.listByLinkId(dto.getWxEnterpriseId(),
                    dto.getEnterpriseId(), linkId);
            dto.setStoreList(storeList);
            return ServiceResponse.success(dto);
        }
        return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "链接不存在");
    }

    @Override
    public ServiceResponse<List<HmLinkDTO>> listForSms(String wxEnterpriseId, String enterprieId, String linkName,
            Integer linkType) {
        return ServiceResponse.success(this.hmLinkService.listForSms(wxEnterpriseId, enterprieId, linkName, linkType));
    }

    @Override
    public ServiceResponse<HmLinkDTO> save(HmLinkDTO dto) {
        return ServiceResponse.success(this.hmLinkService.saveLink(dto, dto.getStoreList()));
    }

    @Override
    public ServiceResponse<Page<HmLinkDTO>> listPage(HmLinkSearchQDTO qdto, BasePageInfo basePageInfo) {
        return ServiceResponse.success(this.hmLinkService.listPage(qdto, basePageInfo));
    }

    @Override
    public ServiceResponse<HmLinkWxaDTO> getLinkHmFromWxa(String enterpriseId, Integer whitchStore, String inStoreId,
            String inClerkId, String linkIdStr, String mixPhone, String unionid) {
        HmQrcodeBO hm = null;
        HmLinkWxaDTO retDTO = new HmLinkWxaDTO();
        log.info("小程序获取活码={},linkId={},mixPhone={}", enterpriseId, linkIdStr, mixPhone);
        Long linkId = Long.parseLong(linkIdStr);
        TabHmLink link = this.hmLinkService.getByLinkId(null, null, linkId);
        String wxEnterpriseId = link.getWxEnterpriseId();
        // 判断导购是否有活码,如果没有返回错误
        if (StringUtils.isNotBlank(inClerkId)) {
            hm = this.getHmFromClerk(inClerkId, wxEnterpriseId);
            if (null == hm) {
                return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "导购活码不存在");
            }
        }
        // 直接返回门店的导购活码,如果没有返回错误
        if (null == hm && StringUtils.isNotBlank(inStoreId)) {
            String customRuleJson = link.getCustomRuleJson();
            log.info("自定义规则={}", customRuleJson);
            JSONObject storeRuleObj = JSONObject.parseObject(customRuleJson);
            int openFlag = storeRuleObj.getIntValue("open");
            if (openFlag == 1) {
                int dz = 0, dy = 0;
                // 1推荐门店 2附件门店 3省市区门店
                int storeType = storeRuleObj.getIntValue("store_type");
                // 省市区
                if (whitchStore == 3) {
                    HmLinkStoreSettingDTO cityStore = storeRuleObj.getObject("city_store", HmLinkStoreSettingDTO.class);
                    dz = cityStore.getDz();
                    dy = cityStore.getDy();
                }
                if (whitchStore == 1 || whitchStore == 2) {
                    HmLinkStoreSettingDTO store = storeRuleObj.getObject("store", HmLinkStoreSettingDTO.class);
                    if (whitchStore == 2 || (storeType == 1 && whitchStore == 1)) {
                        // 附近门店 或者 推荐门店
                        dz = store.getDz();
                        dy = store.getDy();
                    } else if (storeType == 2 && whitchStore == 1) {
                        // 附近门店的推荐门店
                        dz = store.getNdz();
                        dy = store.getNdy();
                    }
                }
                hm = this.getHmFormStore(wxEnterpriseId, inStoreId, dz, dy);
                log.info("获取到的门店配置dz={},dy={},获取的活码={}", dz, dy, JSON.toJSONString(hm));
                if (null == hm) {
                    return ServiceResponse.failure(HaoBanErrCode.ERR_OTHER.getCode(), "导购活码不存在");
                }
            }
        }
        // 会员门店
        if (null == hm && StringUtils.isNotEmpty(mixPhone)) {
            HmLinkStoreSettingDTO fwStore = null;
            HmLinkStoreSettingDTO xgStore = null;
            if (StringUtils.isNotBlank(link.getStoreRuleJson())) {
                String storeRuleJson = link.getStoreRuleJson();
                log.info("会员门店规则={}", storeRuleJson);
                JSONObject storeRuleObj = JSONObject.parseObject(storeRuleJson);
                fwStore = storeRuleObj.getObject("fw_store", HmLinkStoreSettingDTO.class);
                if (null == fwStore || fwStore.getOpen() == 0) {
                    fwStore = null;
                }
                xgStore = storeRuleObj.getObject("xg_store", HmLinkStoreSettingDTO.class);
                if (null == xgStore || xgStore.getOpen() == 0) {
                    xgStore = null;
                }
                String mobile = this.keyDataService.getMobile(mixPhone, linkIdStr);
                log.info("小程序获取活码mobile={}", mobile);
                String memberId = this.memberService.getMemberIdByPhoneNumber(mobile, enterpriseId);
                log.info("小程序获取活码memberId={}", memberId);
                MemberDataDTO member = this.getMemberClerk(enterpriseId, memberId);
                String mainClerkId = member.getMainClerkId();
                String mainStoreId = member.getMainStoreId();
                // 服务门店
                if (null != fwStore) {
                    if (fwStore.getZs() == 1) {
                        hm = this.getHmFromClerk(mainClerkId, wxEnterpriseId);
                    }
                    if (null == hm) {
                        hm = this.getHmFormStore(wxEnterpriseId, mainStoreId, fwStore.getDz(), fwStore.getDy());
                    }
                }
                // 协管门店
                if (null == hm && null != xgStore) {
                    List<MemberStoreClerkDataDTO> list = member.getStoreClerkRel();
                    List<String> storeIdList = new ArrayList<>();
                    // 协管导购
                    for (MemberStoreClerkDataDTO item : list) {
                        String clerkId = item.getClerkIdRel();
                        if (clerkId.equals(mainClerkId)) {
                            continue;
                        }
                        hm = this.getHmFromClerk(clerkId, wxEnterpriseId);
                        if (null != hm) {
                            break;
                        }
                        storeIdList.add(item.getStoreIdRel());
                    }
                    // 店长
                    if (null == hm && xgStore.getDz() == 1) {
                        for (String storeId : storeIdList) {
                            hm = this.getHmFormStore(wxEnterpriseId, storeId, 1, 0);
                            if (null != hm) {
                                break;
                            }
                        }
                    }
                    // 店员
                    if (null == hm && xgStore.getDy() == 1) {
                        for (String storeId : storeIdList) {
                            hm = this.getHmFormStore(wxEnterpriseId, storeId, 0, 1);
                            if (null != hm) {
                                break;
                            }
                        }
                    }
                }
            }
        }
        if (null == hm && StringUtils.isNotBlank(link.getCustomRuleJson())) {
            String customRuleJson = link.getCustomRuleJson();
            log.info("自定义规则={}", customRuleJson);
            JSONObject storeRuleObj = JSONObject.parseObject(customRuleJson);
            int openFlag = storeRuleObj.getIntValue("open");
            if (openFlag == 1) {
                // 门店规则1推荐门店 2附件门店
                int storeType = storeRuleObj.getIntValue("store_type");
                List<HmLinkStoreDTO> storeList = this.hmLinkStoreService.listByLinkId(wxEnterpriseId, enterpriseId,
                        linkId);
                log.info("门店规则={},推荐门店列表={}", storeType, JSON.toJSONString(storeList));
                HmLinkStoreSettingDTO cityStore = storeRuleObj.getObject("city_store", HmLinkStoreSettingDTO.class);
                // 是否开启省市区门店查询
                int selectStoreFlag = cityStore.getOpen();
                log.info("省市区开启={}", selectStoreFlag);
                retDTO.setStoreType(storeType);
                retDTO.setStoreList(storeList);
                retDTO.setSelectStoreFlag(selectStoreFlag);
            }
        }
        if (null != hm) {
            String key = this.keyDataService.saveHmData(enterpriseId, unionid, linkId, hm.getHmId());
        }
        // 生成动态活码
        log.info(JSON.toJSONString(retDTO, true));
        return ServiceResponse.success(retDTO);
    }

    private HmQrcodeBO getHmFromClerk(String clerkId, String wxEnterpriseId) {
        if (StringUtils.isBlank(clerkId) || "-1".equals(clerkId)) {
            return null;
        }
        HmQrcodeBO hm = this.hmQrcodeService.queryByClerkId(clerkId, wxEnterpriseId, 0);
        return hm;
    }

    private HmQrcodeBO getHmFormStore(String wxEnterpriseId, String inStoreId, int dz, int dy) {
        if (StringUtils.isBlank(inStoreId) || "-1".equals(inStoreId)) {
            return null;
        }
        HmQrcodeBO hm = null;
        if (dz == 1) {
            ClerkDTO dzClerk = this.clerkService.getClerkLeaderByStoreId(inStoreId);
            if (null != dzClerk) {
                hm = this.getHmFromClerk(dzClerk.getClerkId(), wxEnterpriseId);
            } else {
                log.info("店长不存在,storeId={}", inStoreId);
            }
        }
        if (null == hm && dy == 1) {
            List<HmQrcodeBO> list = this.hmQrcodeService.listByStoreId(wxEnterpriseId,inStoreId, 0);
            if (CollectionUtils.isNotEmpty(list)) {
                hm = list.get(new Random().nextInt(list.size()));
            }
        }
        return hm;

    }

    // 冗余会员信息
    private MemberDataDTO getMemberClerk(String enterpriseId, String memberId) {
        String returnFields = "memberId,mainClerkId,mainStoreId,storeClerkRel";
        JSONObject memberObj = this.queryDataSingle(enterpriseId, memberId, returnFields);
        if (null == memberObj) {
            return null;
        }
        log.info("会员信息={}", memberObj.toJSONString());
        MemberDataDTO memberDataDTO = memberObj.toJavaObject(MemberDataDTO.class);
        return memberDataDTO;
    }

    public JSONObject queryDataSingle(String enterpriseId, String memberId, String returnFields) {
        JSONObject searchRes = null;
        try {
            JSONObject searchJson = new JSONObject();
            DynamicSearchDTO search = new DynamicSearchDTO();
            searchJson.put("id", memberId);
            search.setSearchJson(searchJson);
            search.setEnterpriseId(enterpriseId);
            search.setColumnCategoryCode("member");
            if (StringUtils.isNotBlank(returnFields)) {
                search.setReturnFileds(returnFields);
            }
            ESResponseQuerySingle single = esApiService.queryDataSingle(search);
            searchRes = single.getRes();
        } catch (Exception e) {
            log.warn("从搜索引擎搜索会员数据异常memberId: {}", memberId, e);
        }
        return searchRes;
    }

}
