package com.gic.haoban.manage.service.service.out.impl.hm;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.PageHelperUtils;
import com.gic.commons.util.UniqueIdUtils;
import com.gic.haoban.base.api.common.errCode.HaoBanErrCodeCommon;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.hm.HmPageQueryDTO;
import com.gic.haoban.manage.api.dto.hm.HmPageRelationQueryDTO;
import com.gic.haoban.manage.api.dto.hm.HmPageRelationRespDTO;
import com.gic.haoban.manage.api.dto.hm.HmPageTemplateDTO;
import com.gic.haoban.manage.api.dto.hm.HmPageTemplateRespDTO;
import com.gic.haoban.manage.api.enums.hm.HmPageTemplateStatus;
import com.gic.haoban.manage.api.service.hm.HmPageTemplateApiService;
import com.gic.haoban.manage.service.dao.mapper.hm.TabHaobanHmPageMapper;
import com.gic.haoban.manage.service.entity.hm.TabHaobanHmPage;
import com.gic.haoban.manage.service.pojo.bo.hm.HmPageRelationBO;
import com.gic.haoban.manage.service.pojo.qo.HmPageRelationQO;
import com.gic.haoban.manage.service.pojo.qo.HmPageTemplateQO;
import com.gic.haoban.manage.service.service.hm.HmPageRelationService;
import com.github.pagehelper.PageHelper;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @Author MUSI
 * @Date 2022/7/3 8:22 PM
 * @Description
 * @Version
 **/
@Component(value = "hmPageTemplateApiServiceImpl")
public class HmPageTemplateApiServiceImpl implements HmPageTemplateApiService {

    private static final Logger log = LoggerFactory.getLogger(HmPageTemplateApiServiceImpl.class);

    private static final String TEMPLATE_CODE_PREFIX = "MB01";

    @Autowired
    private TabHaobanHmPageMapper haobanHmPageMapper;
    @Autowired
    private HmPageRelationService pageRelationService;

    @Override
    public ServiceResponse<Long> saveOrUpdateHmPageTemplate(HmPageTemplateDTO pageTemplateDTO) {

        ServiceResponse<Long> serviceResponse = new ServiceResponse<>();
        if (Objects.isNull(pageTemplateDTO.getPageId())) {
            // save
            TabHaobanHmPage haobanHmPage = new TabHaobanHmPage();
            BeanUtils.copyProperties(pageTemplateDTO, haobanHmPage);
            haobanHmPage.setPageId(UniqueIdUtils.uniqueLong());
            haobanHmPage.setPageCode(this.generatorTemplateCode(haobanHmPage.getPageId()));
            haobanHmPage.setCreateorId(pageTemplateDTO.getClerkId());
            haobanHmPage.setCreateorName(pageTemplateDTO.getClerkName());
            haobanHmPage.setCreateTime(new Date());
            haobanHmPage.setModifierId(pageTemplateDTO.getClerkId());
            haobanHmPage.setModifierName(pageTemplateDTO.getClerkName());
            haobanHmPage.setUpdateTime(new Date());
            haobanHmPage.setStatus(HmPageTemplateStatus.ENABLE.getCode());
            haobanHmPageMapper.insertSelective(haobanHmPage);
            serviceResponse.setResult(haobanHmPage.getPageId());
        } else {
            // update
            TabHaobanHmPage tempPageTemplate = haobanHmPageMapper.selectByPrimaryKey(pageTemplateDTO.getPageId());
            if (Objects.isNull(tempPageTemplate)) {
                // 数据不存在
                serviceResponse.setCode(HaoBanErrCodeCommon.ERR_8.getCode()+"");
                serviceResponse.setMessage(HaoBanErrCodeCommon.ERR_8.getMsg());
                return serviceResponse;
            }
            if (!StringUtils.equals(tempPageTemplate.getEnterpriseId(), pageTemplateDTO.getEnterpriseId())) {
                // 无权限
                serviceResponse.setCode(HaoBanErrCodeCommon.ERR_9.getCode()+"");
                serviceResponse.setMessage(HaoBanErrCodeCommon.ERR_9.getMsg());
                return serviceResponse;
            }
            if (!StringUtils.equals(tempPageTemplate.getWxEnterpriseId(), pageTemplateDTO.getWxEnterpriseId())) {
                // 无权限
                serviceResponse.setCode(HaoBanErrCodeCommon.ERR_9.getCode()+"");
                serviceResponse.setMessage(HaoBanErrCodeCommon.ERR_9.getMsg());
                return serviceResponse;
            }
            TabHaobanHmPage haobanHmPage = new TabHaobanHmPage();
            BeanUtils.copyProperties(pageTemplateDTO, haobanHmPage);
            haobanHmPage.setPageId(tempPageTemplate.getPageId());
            haobanHmPage.setPageCode(null);
            haobanHmPage.setUpdateTime(new Date());
            haobanHmPage.setModifierId(pageTemplateDTO.getClerkId());
            haobanHmPage.setModifierName(pageTemplateDTO.getClerkName());
            haobanHmPageMapper.updateByPrimaryKeySelective(haobanHmPage);
            serviceResponse.setResult(pageTemplateDTO.getPageId());
        }
        return serviceResponse;
    }

    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public ServiceResponse<Boolean> deleteHmPageTemplate(HmPageTemplateDTO pageTemplateDTO) {

        ServiceResponse<Boolean> serviceResponse = new ServiceResponse<>();
        TabHaobanHmPage haobanHmPage = haobanHmPageMapper.selectByPrimaryKey(pageTemplateDTO.getPageId());
        if (Objects.isNull(haobanHmPage) || !Objects.equals(HmPageTemplateStatus.ENABLE.getCode(), haobanHmPage.getStatus())) {
            serviceResponse.setCode(HaoBanErrCodeCommon.ERR_8.getCode()+"");
            serviceResponse.setMessage(HaoBanErrCodeCommon.ERR_8.getMsg());
            return serviceResponse;
        }
        if (checkPageIsBeQuoted(haobanHmPage.getPageCode()) > 0) {
            serviceResponse.setCode(HaoBanErrCodeCommon.ERR_9.getCode()+"");
            serviceResponse.setMessage(HaoBanErrCodeCommon.ERR_9.getMsg());
            return serviceResponse;
        }
        TabHaobanHmPage tempHmPage = new TabHaobanHmPage();
        tempHmPage.setPageId(haobanHmPage.getPageId());
        tempHmPage.setStatus(HmPageTemplateStatus.DISABLE.getCode());
        tempHmPage.setUpdateTime(new Date());
        tempHmPage.setModifierId(pageTemplateDTO.getClerkId());
        tempHmPage.setModifierName(pageTemplateDTO.getClerkName());
        haobanHmPageMapper.updateByPrimaryKeySelective(tempHmPage);
        serviceResponse.setResult(Boolean.TRUE);
        return serviceResponse;
    }

    @Override
    public ServiceResponse<Page<HmPageTemplateRespDTO>> queryPageTemplateList(HmPageQueryDTO pageQueryDTO) {
        HmPageTemplateQO pageTemplateQO = new HmPageTemplateQO();
        BeanUtils.copyProperties(pageQueryDTO, pageTemplateQO);
        pageTemplateQO.setStatus(HmPageTemplateStatus.ENABLE.getCode());
        PageHelper.startPage(pageQueryDTO.getPageNum(), pageTemplateQO.getPageSize());
        com.github.pagehelper.Page<TabHaobanHmPage> tabHaobanHmPages =
                (com.github.pagehelper.Page<TabHaobanHmPage>) haobanHmPageMapper.queryHmPageTemplateList(pageTemplateQO);
        Page<HmPageTemplateRespDTO> pageTemplateRespDTO =
                PageHelperUtils.changePageHelperToCurrentPage(tabHaobanHmPages, HmPageTemplateRespDTO.class);
        ServiceResponse<Page<HmPageTemplateRespDTO>> serviceResponse = new ServiceResponse<>();
        serviceResponse.setResult(pageTemplateRespDTO);
        return serviceResponse;
    }

    /**
     * 校验落地页模板pageTemplateCode 是否被关联
     *
     * @param pageTemplateCode
     * @return true 存在关联
     * false 不存在关联关系
     */
    private Integer checkPageIsBeQuoted(String pageTemplateCode) {
        List<HmPageRelationBO> relations = pageRelationService.queryPageTemplateRelation(pageTemplateCode);
        return 0;
    }

    /**
     * 条件查询落地页关联详情
     *
     * @param relationQueryDTO
     * @return
     */
    @Override
    public ServiceResponse<Page<HmPageRelationRespDTO>> queryPageRelation(HmPageRelationQueryDTO relationQueryDTO) {
        HmPageRelationQO hmPageRelationQo = new HmPageRelationQO();
        hmPageRelationQo.setTemplateSearchText(relationQueryDTO.getSearchText());
        hmPageRelationQo.setEnterpriseId(relationQueryDTO.getEnterpriseId());
        hmPageRelationQo.setWxEnterpriseId(relationQueryDTO.getWxEnterpriseId());
        hmPageRelationQo.setPageNum(relationQueryDTO.getPageNum());
        hmPageRelationQo.setPageSize(relationQueryDTO.getPageSize());
        hmPageRelationQo.setPageCode(relationQueryDTO.getPageId()+"");
        Page<HmPageRelationBO> hmPageRelationPage = pageRelationService.queryPageTemplateRelationWithSearchText(hmPageRelationQo);
        ServiceResponse<Page<HmPageRelationRespDTO>> serviceResponse = new ServiceResponse<>();
        serviceResponse.setResult(PageUtil.changeToCurrentPage(hmPageRelationPage, HmPageRelationRespDTO.class));
        return serviceResponse;
    }

    /**
     * 生成模板code
     * @return
     */
    private String generatorTemplateCode(Long templateId){
        return TEMPLATE_CODE_PREFIX + templateId;
    }

    /**
     * 根据页面id 查询
     *
     * @param templateId
     * @return
     */
    @Override
    public ServiceResponse<HmPageTemplateRespDTO> queryPageTemplateDetail(Long templateId) {
        ServiceResponse<HmPageTemplateRespDTO> serviceResponse = new ServiceResponse<>();
        TabHaobanHmPage haobanHmPage = haobanHmPageMapper.selectByPrimaryKey(templateId);
        if (haobanHmPage == null){
            return serviceResponse;
        }
        HmPageTemplateRespDTO hmPageTemplateRespDTO = new HmPageTemplateRespDTO();
        BeanUtils.copyProperties(haobanHmPage, hmPageTemplateRespDTO);
        serviceResponse.setResult(hmPageTemplateRespDTO);
        return serviceResponse;
    }
}
